#!perl
#===============================================================================
#
# t/02_function.t
#
# DESCRIPTION
#   Test script to check crypt_file() function (and decryption filter).
#
# COPYRIGHT
#   Copyright (C) 2004-2006 Steve Hay.  All rights reserved.
#
# LICENCE
#   You may distribute under the terms of either the GNU General Public License
#   or the Artistic License, as specified in the LICENCE file.
#
#===============================================================================

use 5.006000;

use strict;
use warnings;

use Cwd qw(abs_path);
use File::Spec::Functions qw(canonpath catdir catfile updir);
use FindBin qw($Bin);
use Test::More;

#===============================================================================
# INITIALIZATION
#===============================================================================

my($have_decrypt);

BEGIN {
    my $top_dir = canonpath(abs_path(catdir($Bin, updir())));
    my $lib_dir = catfile($top_dir, 'blib', 'lib', 'Filter', 'Crypto');

    if (-f catfile($lib_dir, 'CryptFile.pm')) {
        require Filter::Crypto::CryptFile;
        Filter::Crypto::CryptFile->import(qw(:DEFAULT $ErrStr));
        plan tests => 289;
    }
    else {
        plan skip_all => 'CryptFile component not built';
    }

    $have_decrypt = -f catfile($lib_dir, 'Decrypt.pm');
}

#===============================================================================
# MAIN PROGRAM
#===============================================================================

MAIN: {
    my $ifile  = 'test.pl';
    my $ofile  = 'test.enc.pl';
    my $iofile = $ifile;
    my $str    = 'Hello, world.';
    my $prog   = qq[print "$str\\n";\n];
    my $head   = 'use Filter::Crypto::Decrypt;';
    my $qrhead = qr/^\Q$head\E/;

    my $perl;
    my $perl_exe = $^X =~ / /o ? qq["$^X"] : $^X;
    if ($] < 5.007003) {
        # Before 5.7.3, -Mblib emitted a "Using ..." message on STDERR, which
        # looks ugly when we spawn a child perl process.
        $perl = qq[$perl_exe -Iblib/arch -Iblib/lib];
    }
    else {
        $perl = qq[$perl_exe -Mblib];
    }

    my($fh, $ifh, $ofh, $iofh, $contents, $saved_contents, $line, $i, $n);

    unlink $ifile or die "Can't delete file '$ifile': $!\n" if -e $ifile;
    unlink $ofile or die "Can't delete file '$ofile': $!\n" if -e $ofile;

    open $fh, ">$ifile" or die "Can't create file '$ifile': $!\n";
    print $fh $prog;
    close $fh;

    open $iofh, "+<$iofile" or die "Can't update file '$iofile': $!\n";
    binmode $iofh;
    ok(crypt_file($iofh), 'crypt_file($fh) returned OK') or
        diag("\$ErrStr = '$ErrStr'");
    close $iofh;

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $iofile});
        is($line, $str, '... and encrypted file runs OK');
    }

    ok(crypt_file($iofile), 'crypt_file($file) returned OK') or
        diag("\$ErrStr = '$ErrStr'");

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted file OK');

    chomp($line = qx{$perl $iofile});
    is($line, $str, '... and decrypted file runs OK');

    open $iofh, "+<$iofile" or die "Can't update file '$iofile': $!\n";
    binmode $iofh;
    ok(crypt_file($iofh, CRYPT_MODE_AUTO()),
       'crypt_file($fh, CRYPT_MODE_AUTO) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $iofh;

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $iofile});
        is($line, $str, '... and encrypted file runs OK');
    }

    ok(crypt_file($iofile, CRYPT_MODE_AUTO()),
       'crypt_file($file, CRYPT_MODE_AUTO) returned OK') or
       diag("\$ErrStr = '$ErrStr'");

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted file OK');

    chomp($line = qx{$perl $iofile});
    is($line, $str, '... and decrypted file runs OK');

    open $iofh, "+<$iofile" or die "Can't update file '$iofile': $!\n";
    binmode $iofh;
    ok(crypt_file($iofh, CRYPT_MODE_ENCRYPT()),
       'crypt_file($fh, CRYPT_MODE_ENCRYPT) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $iofh;

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $iofile});
        is($line, $str, '... and encrypted file runs OK');
    }

    $saved_contents = $contents;

    open $iofh, "+<$iofile" or die "Can't update file '$iofile': $!\n";
    binmode $iofh;
    ok(crypt_file($iofh, CRYPT_MODE_ENCRYPTED()),
       'crypt_file($fh, CRYPT_MODE_ENCRYPTED) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $iofh;

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $saved_contents, '... and left file encrypted');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $iofile});
        is($line, $str, '... and encrypted file still runs OK');
    }

    ok(crypt_file($iofile, CRYPT_MODE_DECRYPT()),
       'crypt_file($file, CRYPT_MODE_DECRYPT) returned OK') or
       diag("\$ErrStr = '$ErrStr'");

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted file OK');

    chomp($line = qx{$perl $iofile});
    is($line, $str, '... and decrypted file runs OK');

    ok(crypt_file($iofile, CRYPT_MODE_DECRYPTED()),
       'crypt_file($file, CRYPT_MODE_DECRYPTED) returned OK') or
       diag("\$ErrStr = '$ErrStr'");

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left file decrypted');

    chomp($line = qx{$perl $iofile});
    is($line, $str, '... and decrypted file still runs OK');

    open $ifh, $ifile or die "Can't read file '$ifile': $!\n";
    binmode $ifh;
    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifh, $ofh), 'crypt_file($fh1, $fh2) returned OK') or
        diag("\$ErrStr = '$ErrStr'");
    close $ofh;
    close $ifh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left input file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ifile;

    open $ofh, $ofile or die "Can't read file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ofh, $ifile), 'crypt_file($fh, $file) returned OK') or
        diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left input file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    chomp($line = qx{$perl $ifile});
    is($line, $str, '... and decrypted output file runs OK');

    unlink $ofile;

    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifile, $ofh), 'crypt_file($file, $fh) returned OK') or
        diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left input file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ifile;

    ok(crypt_file($ofile, $ifile), 'crypt_file($file1, $file2) returned OK') or
        diag("\$ErrStr = '$ErrStr'");

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left input file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    chomp($line = qx{$perl $ifile});
    is($line, $str, '... and decrypted output file runs OK');

    unlink $ofile;

    open $ifh, $ifile or die "Can't read file '$ifile': $!\n";
    binmode $ifh;
    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifh, $ofh, CRYPT_MODE_AUTO()),
       'crypt_file($fh1, $fh2, CRYPT_MODE_AUTO) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;
    close $ifh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left input file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ifile;

    open $ofh, $ofile or die "Can't read file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ofh, $ifile, CRYPT_MODE_AUTO()),
       'crypt_file($fh, $file, CRYPT_MODE_AUTO) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left input file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    chomp($line = qx{$perl $ifile});
    is($line, $str, '... and decrypted output file runs OK');

    unlink $ofile;

    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifile, $ofh, CRYPT_MODE_AUTO()),
       'crypt_file($file, $fh, CRYPT_MODE_AUTO) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left input file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ifile;

    ok(crypt_file($ofile, $ifile, CRYPT_MODE_AUTO()),
       'crypt_file($file1, $file2, CRYPT_MODE_AUTO) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left input file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    chomp($line = qx{$perl $ifile});
    is($line, $str, '... and decrypted output file runs OK');

    unlink $ofile;

    open $ifh, $ifile or die "Can't read file '$ifile': $!\n";
    binmode $ifh;
    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifh, $ofh, CRYPT_MODE_ENCRYPT()),
       'crypt_file($fh1, $fh2, CRYPT_MODE_ENCRYPT) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;
    close $ifh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left intput file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ofile;

    open $ifh, $ifile or die "Can't read file '$ifile': $!\n";
    binmode $ifh;
    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifh, $ofh, CRYPT_MODE_ENCRYPTED()),
       'crypt_file($fh1, $fh2, CRYPT_MODE_ENCRYPTED) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;
    close $ifh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left input file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ifile;

    open $ofh, $ofile or die "Can't read file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ofh, $ifile, CRYPT_MODE_DECRYPT()),
       'crypt_file($fh, $file, CRYPT_MODE_DECRYPT) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left intput file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    chomp($line = qx{$perl $ifile});
    is($line, $str, '... and decrypted output file runs OK');

    unlink $ifile;

    open $ofh, $ofile or die "Can't read file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ofh, $ifile, CRYPT_MODE_DECRYPTED()),
       'crypt_file($fh, $file, CRYPT_MODE_DECRYPTED) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left input file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    chomp($line = qx{$perl $ifile});
    is($line, $str, '... and decrypted output file runs OK');

    unlink $ofile;

    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifile, $ofh, CRYPT_MODE_ENCRYPT()),
       'crypt_file($file, $fh, CRYPT_MODE_ENCRYPT) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left input file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ofile;

    open $ofh, ">$ofile" or die "Can't write file '$ofile': $!\n";
    binmode $ofh;
    ok(crypt_file($ifile, $ofh, CRYPT_MODE_ENCRYPTED()),
       'crypt_file($file, $fh, CRYPT_MODE_ENCRYPTED) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and left input file unencrypted');
    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and encrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ofile});
        is($line, $str, '... and encrypted output file runs OK');
    }

    unlink $ifile;

    ok(crypt_file($ofile, $ifile, CRYPT_MODE_DECRYPT()),
       'crypt_file($file1, $file2, CRYPT_MODE_DECRYPT) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left input file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    chomp($line = qx{$perl $ifile});
    is($line, $str, '... and decrypted output file runs OK');

    unlink $ifile;

    ok(crypt_file($ofile, $ifile, CRYPT_MODE_DECRYPTED()),
       'crypt_file($file1, $file2, CRYPT_MODE_DECRYPTED) returned OK') or
       diag("\$ErrStr = '$ErrStr'");
    close $ofh;

    open $fh, $ofile or die "Can't read file '$ofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and left input file encrypted');
    open $fh, $ifile or die "Can't read file '$ifile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    is($contents, $prog, '... and decrypted output file OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $ifile});
        is($line, $str, '... and decrypted output file runs OK');
    }

    $prog =~ s/\n$//;
    open $fh, ">$iofile" or die "Can't create file '$iofile': $!\n";
    print $fh $prog;
    close $fh;

    ok(crypt_file($iofile), 'file without newline at EOF: OK') or
       diag("\$ErrStr = '$ErrStr'");

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and file encrypted OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $iofile});
        is($line, $str, '... and encrypted file runs OK');
    }

    for ($i = 1; $i <= 16; $i++) {
        open $fh, ">$iofile" or die "Can't create file '$iofile': $!\n";
        binmode $fh;
        print $fh +(';' x ($i - 1)) . "\n";
        close $fh;

        ok(crypt_file($iofile), "$i byte file with newline at EOF: OK") or
            diag("\$ErrStr = '$ErrStr'");

        open $fh, $iofile or die "Can't read file '$iofile': $!\n";
        $contents = do { local $/; <$fh> };
        close $fh;
        like($contents, $qrhead, '... and file encrypted OK');

        SKIP: {
            skip 'Decrypt component not built', 1 unless $have_decrypt;
            chomp($line = qx{$perl $iofile});
            is($line, '', '... and encrypted file runs OK');
        }
    }

    for ($i = 1; $i <= 16; $i++) {
        open $fh, ">$iofile" or die "Can't create file '$iofile': $!\n";
        print $fh ';' x $i;
        close $fh;

        ok(crypt_file($iofile), "$i byte file without newline at EOF: OK") or
            diag("\$ErrStr = '$ErrStr'");

        open $fh, $iofile or die "Can't read file '$iofile': $!\n";
        $contents = do { local $/; <$fh> };
        close $fh;
        like($contents, $qrhead, '... and file encrypted OK');

        SKIP: {
            skip 'Decrypt component not built', 1 unless $have_decrypt;
            chomp($line = qx{$perl $iofile});
            is($line, '', '... and encrypted file runs OK');
        }
    }

    for ($i = 1; $i <= 16; $i++) {
        $str = ';' x $i;
        open $fh, ">$iofile" or die "Can't create file '$iofile': $!\n";
        print $fh qq[print "$str";\n];
        close $fh;

        $n = -s $iofile;
        ok(crypt_file($iofile), "$n byte file with newline at EOF: OK") or
            diag("\$ErrStr = '$ErrStr'");

        open $fh, $iofile or die "Can't read file '$iofile': $!\n";
        $contents = do { local $/; <$fh> };
        close $fh;
        like($contents, $qrhead, '... and file encrypted OK');

        SKIP: {
            skip 'Decrypt component not built', 1 unless $have_decrypt;
            chomp($line = qx{$perl $iofile});
            is($line, $str, '... and encrypted file runs OK');
        }
    }

    for ($i = 1; $i <= 16; $i++) {
        $str = ';' x $i;
        open $fh, ">$iofile" or die "Can't create file '$iofile': $!\n";
        print $fh qq[print "$str";];
        close $fh;

        $n = -s $iofile;
        ok(crypt_file($iofile), "$n byte file without newline at EOF: OK") or
            diag("\$ErrStr = '$ErrStr'");

        open $fh, $iofile or die "Can't read file '$iofile': $!\n";
        $contents = do { local $/; <$fh> };
        close $fh;
        like($contents, $qrhead, '... and file encrypted OK');

        SKIP: {
            skip 'Decrypt component not built', 1 unless $have_decrypt;
            chomp($line = qx{$perl $iofile});
            is($line, $str, '... and encrypted file runs OK');
        }
    }

    $str = ';' x 4096;
    open $fh, ">$iofile" or die "Can't create file '$iofile': $!\n";
    print $fh qq[print "$str";\n];
    close $fh;

    $n = -s $iofile;
    ok(crypt_file($iofile), "$n byte file with newline at EOF: OK") or
        diag("\$ErrStr = '$ErrStr'");

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and file encrypted OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $iofile});
        is($line, $str, '... and encrypted file runs OK');
    }

    $str = ';' x 4096;
    open $fh, ">$iofile" or die "Can't create file '$iofile': $!\n";
    print $fh qq[print "$str";];
    close $fh;

    $n = -s $iofile;
    ok(crypt_file($iofile), "$n byte file without newline at EOF: OK") or
        diag("\$ErrStr = '$ErrStr'");

    open $fh, $iofile or die "Can't read file '$iofile': $!\n";
    $contents = do { local $/; <$fh> };
    close $fh;
    like($contents, $qrhead, '... and file encrypted OK');

    SKIP: {
        skip 'Decrypt component not built', 1 unless $have_decrypt;
        chomp($line = qx{$perl $iofile});
        is($line, $str, '... and encrypted file runs OK');
    }

    unlink $ifile;
    unlink $ofile;
}

#===============================================================================
